/* $Id: plist.c,v 1.5 2002/09/27 14:54:05 aa5779 Exp $ */
/*
	Copyright (C) 2001, 2002 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdlib.h>
#include "util.h"
#include "dstring.h"
#include "lists.h"
#include "interp.h"
#define PLIST_IMP
#include "plist.h"

storage_class plist_storage_class = {plist_create, plist_destroy, 
									 plist_check, plist_get,
									 plist_put, plist_add,
									 plist_remove,
									 plist_first, plist_next,
									 plist_enditer,
									 plist_numofitems,
									 plist_seek
									};

storage *plist_create(interpreter *i, char *name, char *arg)
{
	storage *ns = ALLOC(storage);
	ns->data = NULL;
	ns->class = &plist_storage_class;
	return ns;
}


void plist_destroy(interpreter *i, storage *s)
{
	list_delete(s->data, (list_destroy_t)destroy_html_param, sizeof(html_param));	
}

static int plist_find(html_param *par, dstring *key)
{
	return !ds_compare(par->name, key, NULL, NULL);
}

int plist_check(interpreter *i, storage *s, dstring *key)
{
	return list_find(s->data, key, (list_predicate_t)plist_find) != NULL;
}

dstring *plist_get(interpreter *i, storage *s, dstring *key)
{
	html_param *hp =  list_find(s->data, key, (list_predicate_t)plist_find);
	if(!hp)
		recover_error("no such item in list: %s", DS_BODY(key));
	return hp->val;
}

void plist_put(interpreter *i, storage *s, dstring *key, dstring *val)
{
	html_param *hp =  list_find(s->data, key, (list_predicate_t)plist_find);
	if(!hp)
		recover_error("no such item in list: %s", DS_BODY(key));
	ds_assign(hp->val, val);
}

void plist_add(interpreter *i, storage *s, dstring *key, dstring *val)
{
	html_param *hp = ALLOC(html_param);
	hp->name = ds_fix(ds_copy(key));
	hp->val = ds_fix(ds_copy(val));
	s->data = list_add(s->data, hp);
}

void plist_remove(interpreter *i, storage *s, dstring *key)
{
	list_remove(s->data, key, (list_predicate_t)plist_find, (list_destroy_t)destroy_html_param, sizeof(html_param));
}

dstring *plist_first(interpreter *i, storage *s, void **iter)
{
	if(!s->data)
		return NULL;
	*iter = list_next(s->data);
	return ((html_param *)s->data)->name;
}

dstring *plist_next(interpreter *i, storage *s, void **iter)
{
	html_param *cur;
	if(!*iter)
		return NULL;
	cur = *iter;
	*iter = list_next(*iter);
	return cur->name;
}

void plist_enditer(interpreter *i, storage *s, void **iter)
{
	*iter = NULL;
}

int plist_numofitems(interpreter *i, storage *s)
{
	return list_length(s->data);
}

dstring *plist_seek(interpreter *i, storage *s, void **iter, dstring *key)
{
	*iter = list_find(s->data, key, (list_predicate_t)plist_find);
	if(*iter)
	{
		dstring *name = ((html_param *)*iter)->name;
		*iter = list_next(*iter);
		return name;
	}
	return NULL;
}
